import { Observable } from 'rxjs';
import { IMessageFactory } from './interface/message-factory';
import { IMessageValidator } from './interface/message-validator';
import { IPostmessageAdapter } from './interface/postmessage-adapter';
import { IMessenger, IRequest } from './interface/public-interface';
import { AnyMessage, INotificationObject, IRequestObject, IResponseObject, MappedMessage, MessageType } from './interface/message-objects';
export declare class Messenger implements IMessenger {
    protected readonly messageFactory: IMessageFactory;
    protected readonly messageValidator: IMessageValidator;
    protected readonly adapter: IPostmessageAdapter;
    /**
     * Observable stream of all incoming messages that originate from remoteWindow with a
     * remoteOrigin url matching this.remoteOrigin.
     */
    readonly inboundMessages$: Observable<AnyMessage>;
    /**
     * Filtered subset of inboundMessages$, emitting all messages of type 'notification'
     */
    readonly notifications$: Observable<INotificationObject>;
    /**
     * Filtered subset of inboundMessages$, emitting all messages of type 'request'
     */
    readonly requests$: Observable<IRequestObject>;
    /**
     * Filtered subset of inboundMessages$, emitting all messages of type 'response'
     */
    readonly responses$: Observable<IResponseObject>;
    /**
     * @param {IMessageFactory}     messageFactory   - Factory instance for scalar message objects.
     * @param {IMessageValidator}   messageValidator - Validator instance for incoming messages.
     * @param {IPostmessageAdapter} adapter          - Adapter for interacting with the postMessage API
     */
    constructor(messageFactory: IMessageFactory, messageValidator: IMessageValidator, adapter: IPostmessageAdapter);
    /**
     *
     * Send a request over given channel with given payload. Returns an observable that will
     * emit the response and then complete.
     *
     * @param {string} channel   The channel name of the request.
     * @param {*}      [payload] The payload to send along with the request.
     */
    request<T = any, U = any>(channel: string, payload?: T): Observable<U>;
    /**
     * Send a notification over given channel with given payload.
     *
     * @param {string} channel   - The channel name of the notification.
     * @param {*}      [payload] - The payload to send along with the notification.
     */
    notify<T>(channel: string, payload?: T): void;
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their
     * channel equals given channel name.
     *
     * @param {string} channel - The request channel name to listen to.
     * @return {Observable<object>}
     * @public
     */
    requests<T = any, U = any>(channel: string): Observable<IRequest<T, U>>;
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their
     * channel equals given channel name.
     *
     * @param {string} channel - The notification channel name to listen to.
     * @return {Observable<*>}
     * @public
     */
    notifications<T = any>(channel: string): Observable<T>;
    /**
     * Sends a response through given channel to the remote window, carrying given payload.
     *
     * @param {string} requestId - The request ID of the original request
     * @param {string} channel   - The request-channel name to send the response over
     * @param {*}      payload   - The response payload
     * @return {Messenger}
     * @protected
     */
    protected respond<T>(requestId: string, channel: string, payload: T): this;
    /**
     * Creates an Observable that completes after a single emission of the MessageEvent response for request of given
     * requestId. If no matching response is ever received, the Observable never completes.
     *
     * @param {string} requestId
     * @return {Observable<object>}
     * @protected
     */
    protected createResponseObservable<T>(requestId: string): Observable<T>;
    /**
     * Returns an Observable emitting all inbound messages` of given type.
     *
     * Returns an Observable emitting a subset of MessageEvent objects emitted
     * by this.inboundMessages$, passing through all MessageEvent objects whose
     * data.type property matches given type.
     *
     * @param {string} type
     * @return {Observable<object>}
     * @protected
     */
    protected messagesOfType<T extends MessageType>(type: T): Observable<MappedMessage<T>>;
}
